source /venv/bin/activate 

# pwd
# ls -all
# ls -all img/

# https://stackoverflow.com/questions/51362213/docker-compose-not-printing-stdout-in-python-app
python -u node_demo.py

echo "Paka paka!"

# pwd
# ls -all
# ls -all img/

# tail -f /dev/null